
demo <- function(label="FC3", csvfile="filesAll.csv", intensityBased=0 , deisotoping=1) {

	#label="FC3"; csvfile="filesAll.csv"; intensityBased=0; deisotoping=1;
	params<-init(label, normalizedTemplateSpectrum="", intensityBased, deisotoping)
	dataRead<-readFiles(params, csvfile, sn_cut=1.0)

	dataMatched<-matchPeaks(dataRead$data, dataRead$files, dataRead$matchInit, maxDistance1=0.3, maxDistance2=0.4)
  
	#transpose and make PCA
	rmdata=t(dataMatched$matchedVectors)
	pr = prcomp(rmdata)
	graphics.off()
	plotPCAGraphs(params, dataRead$files, pr, dataRead$filesProperties$filesColourProperty, dataRead$filesProperties$filesCharProperty)
	computeHighestDifference(dataRead$files, dim(dataMatched$matchedVectors)[1], pr)
	plotAbsoluteValueAndRelativeValueChangeGraph(params, dataMatched)
	showComponents(params, dataRead$files, pr, dataMatched$mzVector, dataMatched$matchedVectors, n=3)
}

demoNormalized <- function(label="FC3", csvfile="filesAll.csv", normalizedTemplateSpectrum="FC3-600a.spectrum", intensityBased=0 , deisotoping=1) {

	params<-init(label,normalizedTemplateSpectrum=normalizedTemplateSpectrum, intensityBased,deisotoping)
	dataRead<-readNormalizedFiles(params, csvfile, sn_cut=1.0, normalizeLowMz=900.0, normalizeHighMz=2000.0)

	dataMatched<-matchPeaks(dataRead$data, dataRead$files, dataRead$matchInit, maxDistance1=0.3, maxDistance2=0.4)
  
	#transpose and make PCA
	rmdata=t(dataMatched$matchedVectors)
	pr = prcomp(rmdata)
	graphics.off()
	plotPCAGraphs(params, dataRead$files, pr, dataRead$filesProperties$filesColourProperty, dataRead$filesProperties$filesCharProperty)
	computeHighestDifference(dataRead$files, dim(dataMatched$matchedVectors)[1], pr)
	plotAbsoluteValueAndRelativeValueChangeGraph(params, dataMatched)
	showComponents(params, dataRead$files, pr, dataMatched$mzVector, dataMatched$matchedVectors, n=3)
}

init <- function(name, normalizedTemplateSpectrum="",intensityBased=0, deisotoping=1) {
  
	namef<-name  
	if (normalizedTemplateSpectrum!="") {
		name<-paste(name,"normalized to",normalizedTemplateSpectrum,sep=" ")
		namef<-paste(namef,"normalized to",normalizedTemplateSpectrum,sep=" ")
	}

	if (intensityBased) {
		name=paste(name,"\nintensity based",sep="")
		namef=paste(namef,", intensity based",sep="")
	} else {
		name=paste(name,"\nfwhm x int based",sep="")
		namef=paste(namef,", fwhm x int based",sep="")
	}

	if (deisotoping) {
		name=paste(name,", deisotoped",sep="")
		namef=paste(namef,", deisotoped",sep="")
	} else {
		name=paste(name,", no deisotoping",sep="")
		namef=paste(namef,", no deisotoping",sep="")
	}
	
	cat("name=", name, "\n")
	cat("namef=", namef, "\n")
	cat("intensityBased=", intensityBased, "\n")
	cat("deisotoping=", deisotoping, "\n")
	cat("normalizedTemplateSpectrum=", normalizedTemplateSpectrum, "\n")

	result <- list(intensityBased=intensityBased,deisotoping=deisotoping, name=name, namef=namef, normalizedTemplateSpectrum=normalizedTemplateSpectrum)
	return(result)
}

readNormalizedFiles <- function(params, csvfile="files.csv", sn_cut=1.0, normalizeLowMz=900.0, normalizeHighMz=2000.0) {
	return(readFilesCore(params, csvfile=csvfile, sn_cut=sn_cut, normalize=TRUE, normalizeLowMz=normalizeLowMz, normalizeHighMz=normalizeHighMz))
}

readFiles <- function(params, csvfile="files.csv", sn_cut=1.0) {
	return(readFilesCore(params, csvfile=csvfile, sn_cut=sn_cut, normalize=FALSE))
}

readFilesCore <- function(params, csvfile="files.csv", sn_cut=1.0, normalize=FALSE, normalizeLowMz=900.0, normalizeHighMz=2000.0) {
	
	filesProperties <- read.table(csvfile, header=TRUE, sep=";", dec="," )
	
	files<-array(filesProperties$filesName)

	ratio_medians<-NULL
		
	if (normalize) {
		split4<-strsplit(files, split=".data")
		spectrumFiles<-paste(sapply(split4,"[[",1),".spectrum",sep="")

	
		F1 <- read.table(params$normalizedTemplateSpectrum,header=FALSE)

		mzSpectrumVector<-(floor(normalizeLowMz):ceiling(normalizeHighMz))
		temp_vect<-approx(x=F1$V1,y=F1$V2,xout=mzSpectrumVector,yleft=1.0,yright=1.0)
		a_intensities_norm<-temp_vect$y


		a_intensities<-NULL

		for (i in 1:length(spectrumFiles)) {

			SpectrumFilename <- spectrumFiles[i]
			F1 <- read.table(SpectrumFilename,header=FALSE)
			temp_vect<-approx(x=F1$V1,y=F1$V2,xout=mzSpectrumVector,yleft=1.0,yright=1.0)
			a_intensities<-cbind(a_intensities,temp_vect$y)
	
		}


		ratio<-array(0,cbind(length(a_intensities_norm),length(spectrumFiles)))
		ratio_medians<-array(0,length(spectrumFiles))

		for (j in 1:length(spectrumFiles)) {

			ratio[,j] = a_intensities[,j]/a_intensities_norm
			print(paste("mean(ratio[,",j,"]):",mean(ratio[,j]),", median(ratio[,",j,"]):",median(ratio[,j]),sep=""))
			ratio_medians[j]<-median(ratio[,j])
		}

	} else {
		ratio_medians<-array(1.0,length(files))
	}

	#read and normalize
	
	intensity_offset <- NULL
	sn_base <- NULL
	min_fwhm<-NULL

	Fall<-NULL
	Fall_cut<-NULL

	for (i in 1:length(files)) {

		Filename <- files[i]
		F1 <- read.table(Filename,header=TRUE)
		F1$filename <- Filename

		if (params$deisotoping) {
	
			#peak clusters needn't be continuous, however the peaks that make the discontinuity could be only "None" peaks, i.e. the peaks that are not included in any cluster. It is not allowed, if the peaks of two different clusters are intermixed. When the peak from the other cluster is found, the previous cluster is terminated.
	
			j=1
			store_group="Init group"
			store_mz=0.0
			store_ai=0.0
			store_sn=0.0
			store_integr=0.0
			while (j<=length(F1$mz)) {
				temp_mz=F1$mz[j]
				temp_ai=F1$ai[j]/ratio_medians[i]
				temp_sn=F1$sn[j]
				temp_fwhm=F1$fwhm[j]
				temp_integr=F1$fwhm[j]*F1$ai[j]/ratio_medians[i]
				temp_group=F1$deisotoping_grp[j]
				if (temp_group!="None") {
					if (temp_group!=store_group) {
						if (store_group!="Init group") {
							Fall$mz = c(Fall$mz, store_mz)
							Fall$ai = c(Fall$ai, store_ai)
							Fall$sn = c(Fall$sn, store_sn)
							Fall$filename = c(Fall$filename, Filename)
							Fall$fwhm = c(Fall$fwhm, store_integr/store_ai)
							if (params$intensityBased) {
								Fall$value = c(Fall$value, store_ai)
							} else {
								Fall$value = c(Fall$value, store_integr)
							}
						}
						store_group=temp_group
						store_mz=temp_mz
						store_ai=0.0
						store_sn=0.0
						store_integr=0.0
					}
					store_ai=store_ai+temp_ai
					store_sn=store_sn+temp_sn
					store_integr=store_integr+temp_fwhm*temp_ai
				} else {
					Fall$mz = c(Fall$mz, temp_mz)
					Fall$ai = c(Fall$ai, temp_ai)
					Fall$sn = c(Fall$sn, temp_sn)
					Fall$filename = c(Fall$filename, Filename)
					Fall$fwhm = c(Fall$fwhm, temp_integr/temp_ai)
					if (params$intensityBased) {
						Fall$value = c(Fall$value, temp_ai)
					} else {
						Fall$value = c(Fall$value, temp_integr)
					}
				}
				j=j+1

			}
			if (store_group!="Init group") {
				Fall$mz = c(Fall$mz, store_mz)
				Fall$ai = c(Fall$ai, store_ai)
				Fall$sn = c(Fall$sn, store_sn)
				Fall$filename = c(Fall$filename, Filename)
				Fall$fwhm = c(Fall$fwhm, store_integr/store_ai)
				if (params$intensityBased) {
					Fall$value = c(Fall$value, store_ai)
				} else {
					Fall$value = c(Fall$value, store_integr)
				}
			}

		} else {
			Fall$mz = c(Fall$mz, F1$mz)
			Fall$ai = c(Fall$ai, F1$ai/ratio_medians[i])
			Fall$sn = c(Fall$sn, F1$sn)
			Fall$filename = c(Fall$filename, F1$filename)
			Fall$fwhm = c(Fall$fwhm, F1$fwhm)
			if (params$intensityBased) {
				Fall$value = c(Fall$value, F1$ai/ratio_medians[i])
			} else {
				Fall$value = c(Fall$value, F1$fwhm * F1$ai/ratio_medians[i])
			}

		}

		min_fwhm[i]=min(F1$fwhm)
		intensity_offset[i] = mean(F1$ai-F1$int)/ratio_medians[i]
		sn_base[i] = mean(F1$int/F1$sn)/ratio_medians[i]
		print(paste("Read file:", files[i]))
		flush.console()
	}


	#dataAll = cbind(Fall$mz, Fall$ai, Fall$sn, Fall$fwhm, Fall$value, Fall$filename)
	Fall_cut$mz = Fall$mz[Fall$sn>=sn_cut]
	#Fall_cut$ai = Fall$ai[Fall$sn>=sn_cut]
	#Fall_cut$sn = Fall$sn[Fall$sn>=sn_cut]
	#Fall_cut$fwhm = Fall$fwhm[Fall$sn>=sn_cut]
	Fall_cut$value = Fall$value[Fall$sn>=sn_cut]
	Fall_cut$filename = Fall$filename[Fall$sn>=sn_cut]

	#dataAll_cut = cbind(Fall_cut$mz, Fall_cut$ai, Fall_cut$sn, Fall_cut$fwhm, Fall_cut$value, Fall_cut$filename)
	#dataAll_cut =dataAll_cut[order(as.numeric(dataAll_cut[,1])),]


	data = cbind(Fall_cut$mz, Fall_cut$value, Fall_cut$filename)
	
	if (params$intensityBased) {
		matchInit<-intensity_offset # i.e. sn = 0.0
	} else {
		matchInit<- intensity_offset*min_fwhm # i.e. for sn = 0.0
	}
	
	
	result<-list(data=data, files=files, filesProperties=filesProperties, matchInit=matchInit)
	return(result)

}

matchPeaks<-function(data, files, matchInit, maxDistance1=0.3, maxDistance2=0.4) {

	dataMz<-NULL
	processedMz<-NULL
	aiIndexes<-NULL
	
	#find matching peaks of values
	#match peaks
	
	dataMz = data[order(as.numeric(data[,1])),]
	dataMz = cbind(1:(dim(dataMz)[1]),dataMz)
	processedMz = array(0,dim(dataMz)[1])
	aiIndexes = as.integer(dataMz[order(-1*as.numeric(dataMz[,3])),1])
	
	filecounter = length(files)
	
	res<-NULL
	
	for (i in 1:length(aiIndexes)) {
		if (!processedMz[aiIndexes[i]]) {
	
			line<-c(0,matchInit) # i.e. for sn = 0.0 either intensity_offset or intensity_offset*min_fwhm
	
			#mz representative is in line[1]
			line[1]=as.numeric(dataMz[aiIndexes[i],2])
			line[which(files==dataMz[aiIndexes[i],4])+1]=as.numeric(dataMz[aiIndexes[i],3])
			processedMz[aiIndexes[i]]=1
			
			lowForceProcessed=0
			lowIndex=aiIndexes[i]-1
			if (lowIndex>0) {
				lowDistance=abs(as.numeric(dataMz[aiIndexes[i],2])-as.numeric(dataMz[lowIndex,2]))
			} else {
				lowDistance=maxDistance1+1
			}
			
			highForceProcessed=0
			highIndex=aiIndexes[i]+1
			if (highIndex<=length(aiIndexes)) {
				highDistance=abs(as.numeric(dataMz[aiIndexes[i],2])-as.numeric(dataMz[highIndex,2]))
			} else {
				highDistance=maxDistance1+1
			}
			
			while (min(lowDistance,highDistance)<=maxDistance1) {
				if (lowDistance<=highDistance) {
					if (lowForceProcessed) {
						processedMz[lowIndex]=1
					} else {
						if (processedMz[lowIndex]) {
							lowForceProcessed=1
						} else {
							#we will get maximum from peaks in neighbourhood - in case of double-peak peaks, we will take only the higher one
							line[which(files==dataMz[lowIndex,4])+1]=max(as.numeric(line[which(files==dataMz[lowIndex,4])+1]), as.numeric(dataMz[lowIndex,3]))
							processedMz[lowIndex]=1
						}
					}
					
					
					lowIndex=lowIndex-1
					if (lowIndex>0) {
						lowDistance=abs(as.numeric(dataMz[aiIndexes[i],2])-as.numeric(dataMz[lowIndex,2]))
					} else {
						lowDistance=maxDistance1+1
					}
			
					
				} else {
					if (highForceProcessed) {
						processedMz[highIndex]=1
					} else {
						if (processedMz[highIndex]) {
							highForceProcessed=1
						} else {
							#we will get maximum from peaks in neighbourhood - i.e. think of double-peak peaks
							line[which(files==dataMz[highIndex,4])+1]=max(as.numeric(line[which(files==dataMz[highIndex,4])+1]), as.numeric(dataMz[highIndex,3]))
							processedMz[highIndex]=1
						}
					}
					
					
					highIndex=highIndex+1
					if (highIndex<=length(aiIndexes)) {
						highDistance=abs(as.numeric(dataMz[aiIndexes[i],2])-as.numeric(dataMz[highIndex,2]))
					} else {
						highDistance=maxDistance1+1
					}
				}
			}
			
			res = rbind(res,line)
		}
		if (i%%1000==0) {
			print(paste(i,"indexes processed (of",length(aiIndexes),")"))
			flush.console()
		}
	}
	
	#clean
	dataMz<-NULL
	processedMz<-NULL
	aiIndexes<-NULL
	#end clean
	
	matchedVectors_orig<-NULL
	matchedVectors_orig=array(as.numeric(res), dim=dim(res))
	matchedVectors_orig=matchedVectors_orig[order(matchedVectors_orig[,1]),]
	
	mzVector = matchedVectors_orig[,1]
	matchedVectors<-NULL
	matchedVectors=matchedVectors_orig[,2:(dim(matchedVectors_orig)[2])]
	
	
	minIntensity=apply(matchedVectors,1,min)
	maxIntensity=apply(matchedVectors,1,max)
	
	#make matching if maxima of values differ no more than maxDistance2, but not recursive!
	
	maxIntensity1=maxIntensity[1:length(maxIntensity)-1]
	maxIntensity2=maxIntensity[2:length(maxIntensity)]
	similarIntensity=pmin(maxIntensity1, maxIntensity2)
	dataMaxIntensity=matrix(c(1:(length(mzVector)-1),mzVector[1:length(mzVector)-1],diff(mzVector),similarIntensity),byrow=FALSE,ncol=4)
	
	
	sortedDataMaxIntensity=dataMaxIntensity[dataMaxIntensity[,3]<=maxDistance2,]
	sortedDataMaxIntensity=sortedDataMaxIntensity[order(-1*sortedDataMaxIntensity[,4]),]
	processedIndex=rep(0,dim(matchedVectors_orig)[1])
	
	if ((dim(sortedDataMaxIntensity)[1])>0) {
		for (i in 1:(dim(sortedDataMaxIntensity)[1])) {
			if (processedIndex[sortedDataMaxIntensity[i,1]]==0) {
	
				index1=sortedDataMaxIntensity[i,1]
				index2=sortedDataMaxIntensity[i,1]+1
				if (index2<=(dim(matchedVectors_orig)[1])) {
					if (processedIndex[index2]==0) {
						if (maxIntensity[index2]>maxIntensity[index1]) {
							index2store<-index2
							index2<-index1
							index1<-index2store
						}
						for (j in 2:(dim(matchedVectors_orig)[2])) {
							matchedVectors_orig[index1,j]=max(matchedVectors_orig[index1,j],matchedVectors_orig[index2,j])
						}
						processedIndex[index1]=1
						processedIndex[index2]=2
					}
				}
			}
		}
	}

	matchedVectors_orig2<-NULL
	
	for (i in 1:length(processedIndex)) {
		if (processedIndex[i]<2) {
			matchedVectors_orig2=rbind(matchedVectors_orig2,matchedVectors_orig[i,])
		}
	}
	matchedVectors_orig<-NULL
	
	mzVector = matchedVectors_orig2[,1]
	matchedVectors<-NULL
	matchedVectors=matchedVectors_orig2[,2:(dim(matchedVectors_orig2)[2])]
	
	
	#clean
	processedIndex<-NULL
	dataMaxIntensity<-NULL
	sortedDataMaxIntensity<-NULL
	#end clean
	
	result<-list(mzVector=mzVector, matchedVectors=matchedVectors)
	
	return(result)

}

plotPCAGraphs<-function(params, files, pr, filesColourProperty, filesCharProperty) {


	#select nice colours
	legendcol=NULL
	
	if (length(levels(factor(filesColourProperty)))<=8) {
		legendcolors=c(2:(length(levels(factor(filesColourProperty)))+1),1)
		legendcolors=legendcolors[1:length(levels(factor(filesColourProperty)))]
	} else {
		n=length(levels(factor(filesColourProperty)))
		legendcolors=sample(rainbow(n),size=length(levels(factor(filesColourProperty))),replace=FALSE)
	}
	for (i in 1:length(files)) {
		legendcol=c(legendcol,legendcolors[which(levels(factor(filesColourProperty))==filesColourProperty[i])])
	}
	
	#select nice shapes
	
	legendpch=NULL
	legendchars=c(1:6,15:20,7:14,21:100)
	for (i in 1:length(files)) {
		legendpch=c(legendpch,legendchars[which(levels(factor(filesCharProperty))==filesCharProperty[i])])
	}
	

	#prepair the content of the legend
	
	legendstrings=c("Colours:",paste(levels(factor(filesColourProperty))),"Shapes:",levels(factor(filesCharProperty)))


	legendstringscols=NULL
	legendstringscols=c(legendstringscols,0)
	
	for (i in 1:length(levels(factor(filesColourProperty)))) {
		legendstringscols=c(legendstringscols,legendcolors[which(levels(factor(filesColourProperty))==(levels(factor(filesColourProperty)))[i])])
	}

	legendstringscols=c(legendstringscols,0)

	for (i in 1:length(levels(factor(filesCharProperty)))) {
		legendstringscols=c(legendstringscols,legendcolors[1])
	}


	legendstringspchs=NULL

	legendstringspchs=c(legendstringspchs,0)
	for (i in 1:length(levels(factor(filesColourProperty)))) {
		legendstringspchs=c(legendstringspchs,legendchars[1])
	}

	legendstringspchs=c(legendstringspchs,0)
	for (i in 1:length(levels(factor(filesCharProperty)))) {
		legendstringspchs=c(legendstringspchs,legendchars[which(levels(factor(filesCharProperty))==(levels(factor(filesCharProperty)))[i])])
	}

	#draw the graphs

	windows()
	plot(pr$x, asp=1,col=legendcol, pch=legendpch)
	title(main=paste("PCA (PC1 vs PC2)\n",params$name,sep=""), font.main=4)
	text(pr$x[,1], pr$x[,2],1:length(files), cex=0.6, pos=4, col="black")
	legend("topright",legend=legendstrings,cex=0.8, col=legendstringscols, pch=legendstringspchs, title="Legend")
	

	pdf(paste("PCA (PC1 vs PC2) - ",params$namef,".pdf",sep=""),width=10, height=10)
	plot(pr$x, asp=1,col=legendcol, pch=legendpch)
	title(main=paste("PCA (PC1 vs PC2)\n",params$name,sep=""), font.main=4)
	text(pr$x[,1], pr$x[,2],1:length(files), cex=0.6, pos=4, col="black")
	legend("topright",legend=legendstrings,cex=0.8, col=legendstringscols, pch=legendstringspchs, title="Legend")
	dev.off()
	
	windows()
	plot(pr, main=paste("PCA component variances\n",params$name,sep=""))
	
	pdf(paste("PCA component variances - ",params$namef,".pdf",sep=""),width=10, height=10)
	plot(pr, main=paste("PCA component variances\n",params$name,sep=""))
	dev.off()


}

computeHighestDifference <- function(files, numDim, pr) {

	#computes highest differences between samples in PCA coordinates (PCA1 and PCA2 only, averaged) and distributes it equally between all dimensions 
	highestDifference<-NULL
	for (i in 1:length(files)) {
		highestDifference[i]<-0
		for (j in 1:length(files)) {
			difference = sum(  ((pr$x[i,]-pr$x[j,])^2)[1:2] )
			highestDifference[i]<-max(difference,highestDifference[i])/2
		}
	}
	highestDifference_in_dimension=sqrt(highestDifference/numDim)
	print("highestDifference_in_dimension:")
	print(highestDifference_in_dimension)

}

plotAbsoluteValueAndRelativeValueChangeGraph<-function(params, dataMatched) {



	minIntensity=apply(dataMatched$matchedVectors,1,min)
	maxIntensity=apply(dataMatched$matchedVectors,1,max)
	


	absoluteIntensityChange<-NULL
	absoluteIntensityChange<-(maxIntensity-minIntensity)
	
	i=length(absoluteIntensityChange)
	if (i>25) {
	intensityChangeLimit=(sort(absoluteIntensityChange,decreasing=TRUE))[25]
	} else {
	intensityChangeLimit=0
	}
	
	absoluteIntensityChangeFiltered<-absoluteIntensityChange[absoluteIntensityChange>=intensityChangeLimit]
	absoluteIntensityChangeLabels = round(absoluteIntensityChangeFiltered,1)
	mainLabel<-paste("Absolute value change\n",params$name,sep="")
	windows()
	c<-barplot(t(as.matrix(absoluteIntensityChangeFiltered)),col="yellow",names.arg=round(dataMatched$mzVector[absoluteIntensityChange>=intensityChangeLimit],1), cex.names=0.4, main=mainLabel)
	text(x=c,y=absoluteIntensityChangeFiltered,labels=absoluteIntensityChangeLabels,pos=3,col="blue",cex=0.4)
	
	pdf(paste("Absolute value change graph (",params$namef,").pdf",sep=""),width=10, height=10)
	c<-barplot(t(as.matrix(absoluteIntensityChangeFiltered)),col="yellow",names.arg=round(dataMatched$mzVector[absoluteIntensityChange>=intensityChangeLimit],1), cex.names=0.4, main=mainLabel)
	text(x=c,y=absoluteIntensityChangeFiltered,labels=absoluteIntensityChangeLabels,pos=3,col="blue",cex=0.4)
	dev.off()
	
	relIntensityChangeFiltered<-NULL
	relIntensityChangeFiltered<-(absoluteIntensityChangeFiltered/(minIntensity[absoluteIntensityChange>=intensityChangeLimit]))*100
	mainLabel<-paste("Rel. value change[%]\n",params$name,sep="")
	windows()
	c<-barplot(t(as.matrix(relIntensityChangeFiltered)),col="orange",names.arg=round(dataMatched$mzVector[absoluteIntensityChange>=intensityChangeLimit],1), cex.names=0.4, main=mainLabel)
	text(x=c,y=relIntensityChangeFiltered,labels=round(relIntensityChangeFiltered,1),pos=3,col="blue",cex=0.4)
	
	pdf(paste("Rel. value change[percent] (",params$namef,").pdf",sep=""),width=10, height=10)
	c<-barplot(t(as.matrix(relIntensityChangeFiltered)),col="yellow",names.arg=round(dataMatched$mzVector[absoluteIntensityChange>=intensityChangeLimit],1), cex.names=0.4, main=mainLabel)
	text(x=c,y=relIntensityChangeFiltered,labels=round(relIntensityChangeFiltered,1),pos=3,col="blue",cex=0.4)
	dev.off()

}

showComponents<-function(params, files, pr, mzVector, matchedVectors, n=4) {

	for (i in 1:n) {
		minPC <- min(pr$x[,i])
		minIndexPC <- which(minPC==pr$x[,i])
		print(paste("PC",i," min value:",minPC,", in file:",files[minIndexPC],", file no:",minIndexPC,sep=""))
		maxPC <- max(pr$x[,i])
		maxIndexPC <- which(maxPC==pr$x[,i])
		print(paste("PC",i," max value:",maxPC,", in file:",files[maxIndexPC],", file no:",maxIndexPC,sep=""))
		
		projectedMinPCvector <- (minPC*pr$rotation[,i])+pr$center
		projectedMaxPCvector <- (maxPC*pr$rotation[,i])+pr$center
		
		seePC=cbind(mzVector, pr$center, projectedMinPCvector, projectedMaxPCvector, pr$rotation[,i] ,(projectedMaxPCvector - projectedMinPCvector)*pr$rotation[,i], matchedVectors[,minIndexPC], matchedVectors[,maxIndexPC])
		header = cbind("mz", "center of PCA origin in old coords", paste("minPCAComponent",i," projected sample",sep=""), paste("maxPCAComponent",i," projected sample",sep=""), paste("PCAvector",i," (not normalized)",sep=""), paste("PCAvector",i," influence (i.e. abs(max-min sample)*vector)",sep=""), paste("Existing minPCAcomponent",i," observation (",files[minIndexPC],")",sep=""), paste("Existing maxPCAcomponent",i," observation (",files[maxIndexPC],")",sep=""))
		write.table(seePC,file=paste("PCAcomponent",i," analysis (",params$namef,").csv",sep=""), sep=";", eol="\n", dec=",", col.names=header, row.names=FALSE)
		
		
		
		
		l=dim(seePC)[1]
		if (l>25) {
			influenceLimit=(sort(abs(seePC[,6]),decreasing=TRUE))[25]
		} else {
			influenceLimit=0
		}
		
		seePCfiltered = seePC[abs(seePC[,6])>=influenceLimit,]
		
		
		windows()
		#barplot(t(as.matrix(seePCfiltered[,c(1,5,6)])),beside=TRUE,col=rainbow(3),names.arg=seePCfiltered[,1])
		b<-barplot(t(as.matrix(seePCfiltered[,6])),col="red",names.arg=round(seePCfiltered[,1],1), cex.names=0.4, main=paste("PCAcomponent",i," influence\n",params$name,sep=""))
		text(x=b,y=seePCfiltered[,6],labels=round(seePCfiltered[,6],1),pos=3,col="blue",cex=0.4)
		#text(x=b,y=rep(0,length(seePC1filtered[,6])),labels=round(seePC1filtered[,1],2),pos=3,col="blue",cex=0.5)
		
		pdf(paste("PCAcomponent",i," influence graph (",params$namef,").pdf",sep=""),width=10, height=10)
		b<-barplot(t(as.matrix(seePCfiltered[,6])),col="red",names.arg=round(seePCfiltered[,1],1), cex.names=0.4, main=paste("PCAcomponent",i," influence\n",params$name,sep=""))
		text(x=b,y=seePCfiltered[,6],labels=round(seePCfiltered[,6],1),pos=3,col="blue",cex=0.4)
		dev.off()
		
		seePCvalueChange<-NULL
		seePCvalueChange<-seePCfiltered[,4]-seePCfiltered[,3]
		windows()
		c<-barplot(t(as.matrix(seePCvalueChange)),col="green",names.arg=round(seePCfiltered[,1],1), cex.names=0.4, main=paste("PCAcomponent",i," value change\n",params$name,sep=""))
		text(x=c,y=seePCvalueChange,labels=round(seePCvalueChange,1),pos=3,col="blue",cex=0.4)
		
		pdf(paste("PCAcomponent",i," value change graph (",params$namef,").pdf",sep=""),width=10, height=10)
		c<-barplot(t(as.matrix(seePCvalueChange)),col="green",names.arg=round(seePCfiltered[,1],1), cex.names=0.4, main=paste("PCAcomponent",i," value change\n",params$name,sep=""))
		text(x=c,y=seePCvalueChange,labels=round(seePCvalueChange,1),pos=3,col="blue",cex=0.4)
		dev.off()
		
		windows()
		c<-barplot(t(as.matrix(seePCfiltered[,5])),col="blue",names.arg=round(seePCfiltered[,1],1), cex.names=0.4, main=paste("PCAcomponent",i," factor\n",params$name,sep=""))
		text(x=c,y=seePCfiltered[,5],labels=round(seePCfiltered[,5],2),pos=3,col="green",cex=0.4)
		
		pdf(paste("PCAcomponent",i," factor graph (",params$namef,").pdf",sep=""),width=10, height=10)
		c<-barplot(t(as.matrix(seePCfiltered[,5])),col="blue",names.arg=round(seePCfiltered[,1],1), cex.names=0.4, main=paste("PCAcomponent",i," factor\n",params$name,sep=""))
		text(x=c,y=seePCfiltered[,5],labels=round(seePCfiltered[,5],2),pos=3,col="green",cex=0.4)
		dev.off()
	
	}


}

#functions for faster data processing, read all files first, then filter read data files

prepairNewDataReadFromAllDataRead<-function(csvfile="files.csv", dataReadAll) {
	newFilesProperties<-NULL
	newFilesProperties <- read.table(csvfile, header=TRUE, sep=";", dec="," )

	newFiles<-NULL
	newFiles<-array(newFilesProperties$filesName)

	newDataRead<-NULL
	newDataRead$files<-newFiles
	newDataRead$data<-dataReadAll$data[dataReadAll$data[,3] %in% newFiles,]
	newDataRead$filesProperties<-newFilesProperties
	
	
	#can be permuted so renew the matchInit
	newDataRead$matchInit<-NULL
	for (i in (1:length(newFiles))) {
		newDataRead$matchInit[i]<-dataReadAll$matchInit[which(newFiles[i]==dataReadAll$files)]
	}
	
	return(newDataRead)

}

demoNewDataReadFromAllDataRead<-function(csvfile="files.csv", label="", dataReadAll,n=1) {
	newDataRead<-prepairNewDataReadFromAllDataRead(csvfile, dataReadAll)
	params<-init(label, params$normalizedTemplateSpectrum, params$intensityBased, params$deisotoping)
	dataMatched<-matchPeaks(newDataRead$data, newDataRead$files, newDataRead$matchInit, maxDistance1=0.3, maxDistance2=0.4)
  
	#transpose and make PCA
	rmdata=t(dataMatched$matchedVectors)
	pr = prcomp(rmdata)
	graphics.off()
	plotPCAGraphs(params, newDataRead$files, pr, newDataRead$filesProperties$filesColourProperty, newDataRead$filesProperties$filesCharProperty)
	showComponents(params, newDataRead$files, pr, dataMatched$mzVector, dataMatched$matchedVectors, n)

}

extractFilesPropertiesToCSVfile<-function(pattern=".*\\.data", outputCSVfile="filesAll.csv") {

	files = list.files(ignore.case=TRUE,pattern=pattern)

	split1 <- strsplit(files, split="-")
	split1_1 <- sapply(split1,"[[",1)
	filesColour=gsub("[0-9]","",split1_1,fixed=FALSE)
	filesConcentrations<-sub("[A-Za-z]*","",split1_1,fixed=FALSE)
	filesConcentrations<-sub("$^","0",filesConcentrations)
	
	split1_2 <- sapply(split1,"[[",2)
	split2 <- strsplit(split1_2, split="[abcde]")
	filesAge <- sapply(split2,"[[",1)
	split3 <- strsplit(split1_2, split="[0123456789]*")
	filesRun <- sapply(split3,"[[",2)
	#clean
	split1 <- NULL
	split1_2 <- NULL
	split2 <- NULL
	split3 <- NULL


	blankProperty<-rep("",length(files))
	filesData<-cbind(files, blankProperty, blankProperty, filesColour, filesAge, filesRun, filesConcentrations)
	filesHeader<-cbind("filesName", "filesColourProperty", "filesCharProperty", "filesColour", "filesAge", "filesRun", "filesConcentrations")
	write.table(filesData,file=outputCSVfile, sep=";", eol="\n", dec=",", col.names=filesHeader, row.names=FALSE)

}
